#!/usr/bin/env python

from __future__ import absolute_import

from argparse import ArgumentParser
from os.path import isfile
import sys
import ConfigParser

import logging
import logging.handlers

from .manager import PsqlDumperManager

def parse_args():
    parser = ArgumentParser(description='PsqlDumper - PostgreSQL Dumps Manager')

    parser.add_argument('-c', '--config', dest='config', type=str, nargs='?', default='/usr/local/etc/psqldumper.conf',
                        help="Main configuration file location. Default: '/usr/local/etc/psqldumper.conf'.")
    parser.add_argument('-u', '--pg-user', dest='pg_user', type=str, nargs='?', default='postgres',
                        help="PostgreSQL user to run commands as. Default: 'postgres'.")
    parser.add_argument('-d', '--dump-directory', dest='dump_root', type=str, nargs='?', default='/var/backups/pg_dumps',
                        help="Directory where the dumps will be stored. Default: '/var/backups/pg_dumps'.")
    parser.add_argument('-l', '--logfile', dest='logfile', type=str, nargs='?', default=None,
                        help="Log file location. Default: stdout")
    parser.add_argument('-e', '--email', dest='email', type=str, nargs='?', default=None,
                        help="Email address to send notifications to. No adress means no notifications.")
    parser.add_argument('--notify-success', dest='notify_success', action='store_true',
                        help="Send email notifications for successful dumps too.")
    parser.add_argument('-v', '--verbose', dest='verbose', action='store_true',
                        help="Show progress messages on stdout.")

    return parser, parser.parse_args()


def main():
    argparser, args = parse_args()

    if not isfile(args.config):
        print('Configuration file {} does not exist or is not readable'.format(args.config))
        sys.exit(1)

    if args.logfile:
        logging.getLogger('psqldumper').setLevel(logging.DEBUG)
        handler = logging.handlers.TimedRotatingFileHandler(args.logfile,when='D',backupCount=7)
        FORMAT = "%(asctime)-15s %(message)s"
        fmt = logging.Formatter(FORMAT,datefmt='%Y-%m-%d %H:%M:%S')
        handler.setFormatter(fmt)
        logging.getLogger('psqldumper').addHandler(handler)
    else:
        logging.basicConfig(level=logging.DEBUG, stream=sys.stdout)

    logging.getLogger('psqldumper').info('Loading configuration')
    config = ConfigParser.RawConfigParser()
    config.read(args.config)

    logging.getLogger('psqldumper').info('Starting PsqlDumper manager')
    dumper = PsqlDumperManager(config._sections, args.pg_user, args.dump_root, email=args.email, success=args.notify_success)

    logging.getLogger('psqldumper').info('Checking if we are a PG master')
    if not dumper.check_if_master():
        logging.getLogger('psqldumper').info('Looks like we are a slave, exiting')
        sys.exit(0)

    logging.getLogger('psqldumper').info('Selecting databases for dumping')
    dumper.select_databases()

    logging.getLogger('psqldumper').info('Running dump jobs')
    dumper.run_jobs()

    logging.getLogger('psqldumper').info('Sending notifications')
    dumper.send_notifications()



from __future__ import absolute_import

import logging
import subprocess
import os
import time
import glob

class PsqlDumperManager:

    def __init__(self, conf, pg_user, dump_root, email=None, success=False):
        self.conf = conf
        self.pg_user = pg_user
        self.dump_root = dump_root
        if email:
            self.notify = True
            self.email = email
            self.notify_success = success
        else:
            self.notify = False

    def check_if_master(self):
        try:
            subprocess.check_call("/usr/bin/psql -U {} -t -P format=unaligned -c 'SELECT pg_is_in_recovery();' | grep -q 'f'".format(self.pg_user), shell = True)
            return True
        except subprocess.CalledProcessError:
            return False

    def __db_exists(self, db):
        try:
            subprocess.check_call('/usr/bin/psql -U {} -lqt | cut -d \| -f 1 | grep -w {} > /dev/null'.format(self.pg_user, db), shell = True)
            return True
        except subprocess.CalledProcessError:
            return False

    def select_databases(self):
        for db in self.conf.keys():
            if self.__db_exists(db):
                now = int(time.strftime("%H"))
                freq = int(self.conf[db]['frequency'])
                if now % freq == 0:
                    dump_ext = 'sql' if self.conf[db]['sql_dump'] else 'dmp'
                    dump_args = '' if self.conf[db]['sql_dump'] else ' -Fc -b'

                    self.conf[db]['dump'] = True
                    self.conf[db]['dump_cmd'] = '/usr/bin/pg_dump -U {user}{args} {db} > {root}/{db}_DATE.{ext}'.format(user=self.pg_user, args=dump_args, db=db, root=self.dump_root, ext=dump_ext)
                else:
                    self.conf[db]['dump'] = False
                    self.conf[db]['details'] = 'Not time to backup yet (every {} hours)'.format(self.conf[db]['frequency'])
            else:
                self.conf[db]['dump'] = False
                self.conf[db]['details'] = 'Database does not exist on this server'

    def run_jobs(self):
        for db in self.conf.keys():
            date = time.strftime("%Y%m%d-%H%M%S")

            if self.conf[db]['dump']:
                try:
                    logging.getLogger('psqldumper').info('[{}] Running dump job'.format(db))

                    cmd = '{} 2> {}/{}_{}.log'.format(self.conf[db]['dump_cmd'].replace('DATE',date), self.dump_root, db, date)
                    subprocess.check_call(cmd, shell = True)
                    
                    logging.getLogger('psqldumper').info('[{}] Dump job completed'.format(db))
                    self.conf[db]['status'] = 'Done'

                    logging.getLogger('psqldumper').info('[{}] Cleaning up old dump'.format(db))

                    retention = int(self.conf[db]['retention'])
                    now = time.time()
                    cutoff = now - (retention * 86400)
                    glob_match = '{}/{}_*'.format(self.dump_root, db)
                    for dump in glob.glob(glob_match):
                        c = os.stat(dump).st_ctime
                        if c < cutoff:
                            logging.getLogger('psqldumper').info('[{}] Removing old dump: {}'.format(db, dump))
                            os.remove(dump)
                        else:
                            logging.getLogger('psqldumper').info('[{}] Not touching: {}'.format(db, dump))

                    logging.getLogger('psqldumper').info('[{}] Cleanup done'.format(db))
                except subprocess.CalledProcessError(e):
                    logging.getLogger('psqldumper').info('[{}] Dump job FAILED : {}'.format(db,e))
                    self.conf[db]['status'] = 'Failed'
                    self.conf[db]['details'] = 'Log file is attached'
                    self.conf[db]['logfile'] = '{}_{}.log'.format(db, date)
            else:
                logging.getLogger('psqldumper').info('[{}] Database dump ignored: {}'.format(db, self.conf[db]['details']))
                self.conf[db]['status'] = 'Ignored'

    def send_notifications(self):
        pass



import os
from setuptools import setup, find_packages

try:
    README = open(os.path.join(os.path.dirname(__file__), 'README.md')).read()
except IOError:
    README = ''

setup(
    name = 'psqldumper',
    version = '0.1.4',
    author = 'Nicolas Brisac',
    author_email = 'nbrisac@oasiswork.fr',
    description = 'Postgresql dump management helper.',
    license = 'MIT',
    url = 'https://git.owk.cc/nbrisac/psqldumper',
    packages = find_packages(),
    namespace_packages = ['psqldumper'],
    install_requires = [],
    extras_require = {},
    entry_points = {'console_scripts': [
        'psqldumper = psqldumper.main:main',
    ]},
    package_data = {'psqldumper': ['etc/*.conf.sample']},
    classifiers = [])
